import json

from twisted.mail.alias import handle

from dspt_api.util.general.handle_redis import get_data_from_redis
from dspt_api.util.general.handle_seat import general_seat_params
from dspt_api.util.general.get_order_num import general_order_num
from dspt_api.util.general.handle_goods import general_goods_params
from dspt_api.util.general.handle_ecard import get_ecard
import time
import random


class ApiSeatLockBuy:
    def __init__(self, **kwargs):
        self.member_type = kwargs.get('member_type')
        self.api = kwargs.get('api')
        self.ip = kwargs.get('ip')
        self.env = kwargs.get('env')
        self.cid = kwargs.get('cid')
        self.pid = kwargs.get('pid')
        self.sale_type = kwargs.get('sale_type')
        self.pay_type = kwargs.get('pay_type')
        self.user_data = {
            'ip': self.ip,
            'env': self.env,
            'member_type': self.member_type,
            'pid': self.pid,
            'cid': self.cid,
        }
        self.return_data = []

    def get_suggestion(self):
        print('get_suggestion', self.user_data, self.sale_type, self.pay_type)
        # 通用字段
        self.add_param('mobile', '18688886666', timestamp=1000000000000)
        _partner_buy_ticket_id = general_order_num('BUY_T', self.api, self.member_type, self.ip, self.env, self.pid,
                                                   self.cid)
        self.add_param('partner_buy_ticket_id', _partner_buy_ticket_id)

        # 联名卡卡号
        ecard_num = None
        if self.pay_type == 'ecard':
            ecard_num = get_ecard(self.ip, self.env, self.pid, self.cid)
            self.add_param('ecard_number', ecard_num, timestamp=1000000000000)

        self.handle()
        print('ApiSeatLockBuy-get_suggestion', self.return_data)
        return self.return_data

    @staticmethod
    def get_timestamp():
        return int(time.time() * 1000)

    def handle(self):
        # 处理影票部分
        _ticket_result, _seat, _ticket_cash, _play_id, _play_update_time, _lock_flag, _quan, _ticket_timestamp = general_seat_params(
            self.pay_type, self.user_data)
        print('general_seat_params', _ticket_result, _seat, _ticket_cash, _play_id, _play_update_time, _lock_flag,
              _quan, _ticket_timestamp)
        if _ticket_result is False:
            self.add_param('play_id', '请手动输入参数，或先请求接口：3.1.5 获取放映计划列表', timestamp=_ticket_timestamp)
            self.add_param('seat', '请手动输入参数，或先请求接口：3.1.8 获取某场次座位状态', timestamp=_ticket_timestamp)
            self.add_param('lock_flag', '请手动输入参数，或先请求接口：3.3.1 座位锁定', timestamp=_ticket_timestamp)
            self.add_param('play_update_time', '请手动输入参数，或先请求接口：3.1.5 获取放映计划列表',
                           timestamp=_ticket_timestamp)
            self.add_param('cash', '请手动输入参数，或先请求接口：3.1.5 获取放映计划列表', timestamp=_ticket_timestamp)
            if self.pay_type == 'ecard':
                self.add_param('ecard_number', '请手动输入参数，检查相关接口返回值：3.4.6 获取座位的联名卡价格',
                               timestamp=_ticket_timestamp)
            if self.pay_type == 'yushouquan':
                self.add_param('coupons', '3.6.1添加券接口中选择的券类型为扫码券，与当前设置不符合',
                               timestamp=_ticket_timestamp)
            if self.pay_type == 'equan':
                self.add_param('card_coupons', '3.6.1添加券接口中选择的券类型为扫码券，与当前设置不符合',
                               timestamp=_ticket_timestamp)
            return False
        self.add_param('play_id', _play_id, timestamp=_ticket_timestamp)
        self.add_param('seat', _seat, timestamp=_ticket_timestamp)
        self.add_param('lock_flag', _lock_flag, timestamp=_ticket_timestamp)
        self.add_param('play_update_time', _play_update_time, timestamp=_ticket_timestamp)
        # 处理卖品部分
        if self.sale_type == 'all':
            delivery_appoint_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time() + 5 * 60))
            _good_result, _goods_field, _goods_cash, _quan, _goods_timestamp = general_goods_params(self.pay_type,
                                                                                                    self.user_data,
                                                                                                    _ticket_cash)
            print('_goods_field', _goods_field)
            if _good_result is False:
                self.add_param('goods', '请手动输入参数，或先请求接口：3.2.1 获取卖品列表', timestamp=_goods_timestamp)
                self.add_param('cash', '请手动输入参数，或先请求接口：3.2.1 获取卖品列表', timestamp=_goods_timestamp)
                return False
            self.add_param('goods', json.dumps(_goods_field), timestamp=_goods_timestamp)
            self.add_param('cash', _goods_cash, timestamp=_goods_timestamp)
            self.add_param('delivery_type', '2', is_checked=False, timestamp=_goods_timestamp)
            self.add_param('delivery_location', '1号厅', is_checked=False, timestamp=_goods_timestamp)
            self.add_param('delivery_appoint_time', delivery_appoint_time, is_checked=False, timestamp=_goods_timestamp)
            self.add_param('contact_number', '18688886666', is_checked=False, timestamp=_goods_timestamp)
        else:
            self.add_param('cash', _ticket_cash, timestamp=_ticket_timestamp)
        # 处理券码部分
        if self.pay_type == 'yushouquan':
            self.add_param('coupons', _quan, timestamp=_ticket_timestamp)
        if self.pay_type == 'equan':
            self.add_param('card_coupons', _quan, timestamp=_ticket_timestamp)

    def add_param(self, field, value, is_checked=True, result=True, timestamp=int(time.time() * 1000)):
        self.return_data.append(
            {'param': field, 'value': value, 'is_checked': is_checked, 'result': result, 'timestamp': timestamp})
