import pymysql
from pymysql.cursors import DictCursor
from mock.models import ZZMockModel
from mock.cinema_sql import *


# 转换sessionCode为show_id
def handle_session_code(session_code):
    while session_code.startswith('0'):
        session_code = session_code[1:]
    return session_code


def get(cinema_data, request):
    # 获取审核状态
    audit_config = ZZMockModel.objects.filter(ip=cinema_data.ip).first()
    target_show_id = audit_config.auditShowId
    target_status = audit_config.auditStatus
    # 获取请求数据
    session_code = request.GET.get('sessionCode')
    overtime_type = request.GET.get('overtimeType')
    req_show_id = handle_session_code(session_code)

    print(target_show_id, target_status, session_code, overtime_type)
    return handler(cinema_data, target_show_id, target_status, req_show_id, overtime_type)


def get_json(cinema_data, request):
    # type_dict = {
    #     '1':
    #     '2':
    # }
    # 获取请求参数
    target_show_id = req_show_id = request.GET.get('show')
    overtime_type = request.GET.get('type')
    target_status = request.GET.get('status')

    print(target_show_id, target_status, req_show_id, overtime_type)
    return handler(cinema_data, target_show_id, target_status, req_show_id, overtime_type)


def handler(cinema_data, target_show_id, target_status, req_show_id, overtime_type):
    # 审核状态映射
    # 退票状态 1-未提交 2-审核中 3-审核通过 4-审核失败
    # 售票状态 1-已审核 2-已驳回 3-审核中 4-已补登 5-未提交 6-提交失败
    # 专资接口 # operation 1售票 2退票 # status 0不通过 1通过 2待审核 99待提交
    refund_status_dict = {
        '3': '1',
        '4': '0'
    }
    sell_status_dict = {
        '1': '1',
        '2': '0',
        '4': '1',
    }

    # 定义影院数据库
    db_config = {
        'host': cinema_data.ip,
        'user': cinema_data.db_user,
        'password': cinema_data.db_pwd,
        'database': 'cine',
        'connect_timeout': 5,
    }

    # 如果不是要测试的场次直接返回None, 从而执行bypass
    if req_show_id != target_show_id:
        print('不相同')
        return None

    db_conn = pymysql.Connect(**db_config)
    db_cursor = db_conn.cursor(cursor=DictCursor)
    ticket_list = []
    # 处理过场售票

    if overtime_type == '0':
        # zz_audit_status 专资审核状态 1-已审核 2-已驳回 3-审核中 4-已补登 5-未提交 6-提交失败
        # cinema_sell_add_status 补登状态 1-已审核 2-已驳回 3-审核中,
        db_cursor.execute(GET_OVERTIME_SELL_TICKET_SQL, (target_show_id,))
        sell_tickets = db_cursor.fetchall()
        for ticket in sell_tickets:
            sell_status = str(ticket['audit_status'])
            if sell_status in ('5', '6'):
                continue
            elif sell_status == '3':
                ticket_list.append({
                    'ticketNo': ticket['ticket_no'],
                    'operation': 1,
                    'status': target_status
                })
            else:
                ticket_list.append({
                    'ticketNo': ticket['ticket_no'],
                    'operation': 1,
                    'status': sell_status_dict[sell_status]
                })
    # 处理过场退票
    if overtime_type == '1':
        # cinema_refund_order_detail
        db_cursor.execute(GET_OVERTIME_REFUND_TICKET_SQL, (target_show_id,))
        refund_tickets = db_cursor.fetchall()

        for ticket in refund_tickets:
            refund_status = str(ticket['refund_status'])
            if refund_status == '1':
                continue
            elif refund_status == '2':
                ticket_list.append({
                    'ticketNo': ticket['ticket_no'],
                    'operation': 2,
                    'status': target_status
                })
            else:
                ticket_list.append({
                    'ticketNo': ticket['ticket_no'],
                    'operation': 2,
                    'status': refund_status_dict[refund_status]
                })
    print('ticket_list', ticket_list)
    mock_data = {'message': '操作成功',
                 'data': {'ticketList': ticket_list},
                 'code': '200',
                 'status': 'success'}
    return mock_data
